"""Slices BI Images API."""

from collections.abc import Collection
from datetime import datetime
from typing import Annotated

from httpx import Client, HTTPError
from msgspec import Meta, Struct
from msgspec.json import Decoder as JsonDecoder

from .exceptions import from_httpx_exception
from .pagination import PagedCollection


class VMFlavor(Struct, tag_field="flavor_type", tag="vm"):
    """VM Flavor."""

    created_at: Annotated[datetime, Meta(tz=True)]
    "Timestamp when this flavor was created"
    description: str
    "Description of this flavor"
    disk_mb: int
    """The size of the root disk that will be created in MB.
    If 0 the root disk will be set to exactly the size of the image used to deploy the instance."""

    friendly_name: str
    "User readable ID"
    id: str
    "TypeID uniquely identifying a resource"
    ram_mib: int
    "The amount of RAM in MiB"
    vcpus: int
    "The number of virtual CPUs that will be allocated to the server"


class BareMetalFlavor(Struct, tag_field="flavor_type", tag="baremetal"):
    """Bare metal flavor."""

    created_at: Annotated[datetime, Meta(tz=True)]
    "Timestamp when this flavor was created"
    description: str
    "Description of this flavor"
    disk_mb: int
    """The size of the root disk that will be created in MB.
    If 0 the root disk will be set to exactly the size of the image used to deploy the instance."""

    friendly_name: str
    "User readable ID"
    id: str
    "TypeID uniquely identifying a resource"
    ram_mib: int
    "The amount of RAM in MiB"

    cpu_cores: int
    "CPU cores (per CPU socket)"
    cpu_model: str
    "CPU model."
    cpu_sockets: int
    "CPU count (= CPU socket count)"
    cpu_threads: int
    "CPU Threads (per CPU core)"
    gpu_model: str | None
    network_interface_count: int
    "Number of test network interfaces present. (There is always at least the 'control' interface)"


AnyFlavor = VMFlavor | BareMetalFlavor
_flavor_decoder = JsonDecoder(AnyFlavor)


class BiFlavorClient:
    """Basic Infrastructure Flavor API Client."""

    API_BASE_PATH = "/apis/bi.slices.eu/v1alpha2/flavors/"

    def __init__(self, client: Client) -> None:
        """Create Basic Infrastructur Flavor Client."""
        self._client = client

        assert client.base_url is not None, "Expected base_url to be set"

    def get(self, id: str) -> AnyFlavor:
        """Get flavor.

        :param id: Flavor ID
        """
        response = self._client.get(f"{self.API_BASE_PATH}{id}")

        try:
            response.raise_for_status()
            return _flavor_decoder.decode(response.content)
        except HTTPError as err:
            raise from_httpx_exception(err) from err

    def delete(self, flavor: AnyFlavor) -> AnyFlavor:
        """Delete flavor.

        :param flavor: Flavor
        :return: Deleted flavor
        """
        return self.delete_by_id(flavor.id)

    def delete_by_id(self, id: str) -> AnyFlavor:
        """Delete flavor.

        :param id: Flavor ID
        :return: Deleted flavor
        """
        response = self._client.delete(f"{self.API_BASE_PATH}{id}")

        try:
            response.raise_for_status()
            return _flavor_decoder.decode(response.content)
        except HTTPError as err:
            raise from_httpx_exception(err) from err

    def list(self) -> Collection[AnyFlavor]:
        """List all flavors."""

        return PagedCollection("flavor", AnyFlavor, self.API_BASE_PATH, self._client)
